# Introduction
MultiQC is a reporting tool that parses summary statistics from results and log files
generated by other bioinformatics tools. MultiQC doesn't run other tools for you -
it's designed to be placed at the end of analysis pipelines or to be run manually
when you've finished running your tools.

When you launch MultiQC, it recursively searches through any provided file paths
and finds files that it recognises. It parses relevant information from these and
generates a single stand-alone HTML report file. It also saves a directory of data
files with all parsed data for further downstream use.

# Installing MultiQC

## Installing Python
To see if you have python installed, run `python --version` on the command line.
If you see version 2.7+, 3.4+ or 3.5+ then you can skip this step.

We recommend using virtual environments to manage your Python installation.
Our favourite is Anaconda, a cross-platform tool to manage Python environments.
You can installation instructions for Anaconda
[here](http://conda.pydata.org/docs/install/quick.html).

Once Anaconda is installed, you can create an environment with the
following commands:

```
conda create --name py2.7 python=2.7
source activate py2.7
# Windows: activate py2.7
```

You'll want to add the `source activate py2.7` line to your `.bashrc` file so
that the environment is loaded every time you load the terminal.

## Installing with conda
If you're using `conda` as described above, you can install MultiQC from the `bioconda`
channel as follows:
```
conda install -c bioconda multiqc
```

## Installation with pip
This is the easiest way to install MultiQC. `pip` is the package manager for
the Python Package Manager. It comes bundled with recent versions of Python,
otherwise you can find installation instructions [here](http://pip.readthedocs.org/en/stable/installing/).

You can now install MultiQC from
[PyPI](https://pypi.python.org/pypi/multiqc) as follows:
```
pip install multiqc
```

If you would like the development version, the command is:
```
pip install git+https://github.com/ewels/MultiQC.git
```

Note that if you have problems with read-only directories, you can install to
your home directory with the `--user` parameter (though it's probably better
to use virtual environments, as described above).
```
pip install --user multiqc
```

## Manual installation
If you'd rather not use either of these tools, you can clone the code and install the code yourself:
```
git clone https://github.com/ewels/MultiQC.git
cd MultiQC
python setup.py install
```

`git` not installed? No problem - just download the flat files:
```
curl -LOk https://github.com/ewels/MultiQC/archive/master.zip
unzip master.zip
cd MultiQC-master
python setup.py install
```

## Updating MultiQC
You can update MultiQC from [PyPI](https://pypi.python.org/pypi/multiqc)
at any time by running the following command:
```
pip --update multiqc
```

To update the development version, use:
```
pip install --force git+https://github.com/ewels/MultiQC.git
```

If you cloned the `git` repo, just pull the latest changes and install:
```
cd MultiQC
git pull
python setup.py install
```

If you downloaded the flat files, just repeat the installation procedure.

## Installing as an environment module
Many people using MultiQC will be working on a HPC environment.
Every server / cluster is different, and you're probably best off asking
your friendly sysadmin to install MultiQC for you. However, with that
in mind, here are a few general tips for installing MultiQC into an
environment module system:

MultiQC comes in two parts - the `multiqc` python package and the
`multiqc` executable script. The former must be available in `$PYTHONPATH`
and the script must be available on the `$PATH`.

A typical installation procedure with an environment module Python install
might look like this: _(Note that `$PYTHONPATH` must be defined before `pip` installation.)_
```
VERSION=0.7
INST=/path/to/software/multiqc/$VERSION
module load python/2.7.6
mkdir $INST
export PYTHONPATH=$INST/lib/python2.7/site-packages
pip install --install-option="--prefix=$INST" multiqc
```

Once installed, you'll need to create an environment module file.
Again, these vary between systems a lot, but here's an example:

```bash
#%Module1.0#####################################################################
##
## MultiQC
##

set components [ file split [ module-info name ] ]
set version [ lindex $components 1 ]
set modroot /path/to/software/multiqc/$version

proc ModulesHelp { } {
    global version modroot
    puts stderr "\tMultiQC - use MultiQC $version"
    puts stderr "\n\tVersion $version\n"
}
module-whatis   "Loads MultiQC environment."

# load required modules
module load python/2.7.6

# only one version at a time
conflict multiqc

# Make the directories available
prepend-path    PATH        $modroot/bin
prepend-path	PYTHONPATH	$modroot/lib/python2.7/site-packages
```

## Using the Docker container
A Docker container based on `python:2.7-slim` is provided.
Specify the volume to bind mount as desired with `-v`, same for the working directory inside the container with `-w`. Or just use `-v "$PWD":"$PWD" -w "$PWD"` to run in current directory.
For more information, look into [the Docker documentation](https://docs.docker.com/engine/reference/commandline/run/)

The usual multiqc command line should work fine:
```
docker run -v "$PWD":"$PWD" -w "$PWD" ewels/multiqc multiqc .
```


## Using MultiQC through Galaxy

### On the main Galaxy instance
The easiest and fast manner to use MutliQC is to use the [usegalaxy.org](https://usegalaxy.org/) main Galaxy instance where you will find [MultiQC Galaxy tool](https://usegalaxy.org/?tool_id=toolshed.g2.bx.psu.edu%2Frepos%2Fengineson%2Fmultiqc%2Fmultiqc%2F1.0.0.0&version=1.0.0.0&__identifer=2sjdq8d9r3l) under the *NGS: QC and manipualtion* tool panel section.

### On your instance
You can install MultiQC on your own Galaxy instance through your Galaxy admin space, searching on the [main Toolshed](https://toolshed.g2.bx.psu.edu/) for the [MultiQC repository](https://toolshed.g2.bx.psu.edu/view/iuc/multiqc/3bad335ccea9) available under the *visualization*, *statistics* and *Fastq Manipulation* sections.
